"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createStub = createStub;
exports.stubMethod = stubMethod;
var current = 0;
var restores = [];
beforeEach(function () {
  return current++;
});
afterEach(function () {
  restores.forEach(function (restore) {
    return restore();
  });
  restores = [];
});

function assertActive(test) {
  if (test !== current) {
    throw new Error('Stubs cannot be reused after the test that created them');
  }
}

function defineGetter(test, stub, methodName, method) {
  Object.defineProperty(stub, methodName, {
    get: function get() {
      assertActive(test);
      return method();
    }
  });
}

function createStub() {
  var test = current;
  var called = false;
  var callCount = 0;
  var calls = [];

  function stub() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    assertActive(test);
    called = true;
    callCount++;
    calls.push({
      context: this,
      args: args
    });
  }

  defineGetter(test, stub, 'called', function () {
    return called;
  });
  defineGetter(test, stub, 'callCount', function () {
    return callCount;
  });
  defineGetter(test, stub, 'calls', function () {
    return calls;
  });
  return stub;
}

function stubMethod(obj, methodName, method) {
  var test = current;
  var prev = obj[methodName];
  var called = false;
  var callCount = 0;
  var calls = [];
  var restored = false;

  obj[methodName] = function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    assertActive(test);
    called = true;
    callCount++;
    calls.push({
      context: this,
      args: args
    });
    method.apply(this, args);
  };

  var stub = {};
  defineGetter(test, stub, 'called', function () {
    return called;
  });
  defineGetter(test, stub, 'callCount', function () {
    return callCount;
  });
  defineGetter(test, stub, 'calls', function () {
    return calls;
  });

  function restore() {
    if (!restored) {
      obj[methodName] = prev;
      restored = true;
    }
  }

  stub.restore = function () {
    assertActive(test);
    restore();
  };

  restores.push(restore);
  return stub;
}